/*
 * CPPFile1.cpp
 *
 * Created: 4/15/2017 9:22:11 PM
 *  Author: auty1
 */ 

#include "Arduino.h"
#include "config.h"


#include "Wire.h"
#include "wiring_private.h" // pinPeripheral() function
#include "magnetometer.h"
TwoWire myWire(&sercom2, I2C_MAG_D, I2C_MAG_C);

bool Magnetometer::begin()
{
	myWire.begin();
pinPeripheral(3, PIO_SERCOM_ALT);
pinPeripheral(4, PIO_SERCOM_ALT);

delay(100);
	if ((fastRegister8(HMC5883L_REG_IDENT_A) != 0x48)
	|| (fastRegister8(HMC5883L_REG_IDENT_B) != 0x34)
	|| (fastRegister8(HMC5883L_REG_IDENT_C) != 0x33))
	{
		return false;
	}

	setRange(HMC5883L_RANGE_1_9GA);
	setMeasurementMode(HMC5883L_CONTINOUS);
	setDataRate(HMC5883L_DATARATE_15HZ);
	setSamples(HMC5883L_SAMPLES_4);

	mgPerDigit = 0.92f;

	return true;
}

Vector Magnetometer::readRaw(void)
{
	v.XAxis = readRegister16(HMC5883L_REG_OUT_X_M);
	v.YAxis = readRegister16(HMC5883L_REG_OUT_Y_M);
	v.ZAxis = readRegister16(HMC5883L_REG_OUT_Z_M);

	return v;
}

Vector Magnetometer::readNormalize(void)
{
	v.XAxis = ((float)readRegister16(HMC5883L_REG_OUT_X_M)) * mgPerDigit;
	v.YAxis = ((float)readRegister16(HMC5883L_REG_OUT_Y_M)) * mgPerDigit;
	v.ZAxis = (float)readRegister16(HMC5883L_REG_OUT_Z_M) * mgPerDigit;

	return v;
}

float Magnetometer::getHeading(void)
{
	
	Vector v = readNormalize();
	float h;
	//if(v.YAxis>0)
	//{
		//h = 90.0 -atan2(v.XAxis,v.YAxis)*180.0/PI;
	//}
	//else if(v.YAxis<0)
	//{
		//
		//h = 270 - atan2(v.XAxis,v.YAxis)*180.0/PI;
	//}
	//else if(v.YAxis == 0)
	//{
		//h = (v.XAxis>0)? 0.0 : 180.0;
	//}
	//
	h = atan2(v.XAxis, v.YAxis)*180/PI;
	h += declination;
	DPRINT("Read Heading to be "); DPRINT(h); DPRINT(" degrees\n");
				DPRINT("Read X "); DPRINT(-v.XAxis); DPRINT("\n");
				DPRINT("Read Y "); DPRINT(-v.YAxis); DPRINT("\n");
				DPRINT("Read Z "); DPRINT(-v.ZAxis); DPRINT("\n");
	return h;
}

void Magnetometer::setDeclination(float dec)
{
	declination = dec;
}

float Magnetometer::getDeclination()
{
	
	return declination;
}

void Magnetometer::setRange(hmc5883l_range_t range)
{
	switch(range)
	{
		case HMC5883L_RANGE_0_88GA:
		mgPerDigit = 0.073f;
		break;

		case HMC5883L_RANGE_1_3GA:
		mgPerDigit = 0.92f;
		break;

		case HMC5883L_RANGE_1_9GA:
		mgPerDigit = 1.22f;
		break;

		case HMC5883L_RANGE_2_5GA:
		mgPerDigit = 1.52f;
		break;

		case HMC5883L_RANGE_4GA:
		mgPerDigit = 2.27f;
		break;

		case HMC5883L_RANGE_4_7GA:
		mgPerDigit = 2.56f;
		break;

		case HMC5883L_RANGE_5_6GA:
		mgPerDigit = 3.03f;
		break;

		case HMC5883L_RANGE_8_1GA:
		mgPerDigit = 4.35f;
		break;

		default:
		break;
	}

	writeRegister8(HMC5883L_REG_CONFIG_B, range << 5);
}

hmc5883l_range_t Magnetometer::getRange(void)
{
	return (hmc5883l_range_t)((readRegister8(HMC5883L_REG_CONFIG_B) >> 5));
}

void Magnetometer::setMeasurementMode(hmc5883l_mode_t mode)
{
	uint8_t value;

	value = readRegister8(HMC5883L_REG_MODE);
	value &= 0b11111100;
	value |= mode;

	writeRegister8(HMC5883L_REG_MODE, value);
}

hmc5883l_mode_t Magnetometer::getMeasurementMode(void)
{
	uint8_t value;

	value = readRegister8(HMC5883L_REG_MODE);
	value &= 0b00000011;

	return (hmc5883l_mode_t)value;
}

void Magnetometer::setDataRate(hmc5883l_dataRate_t dataRate)
{
	uint8_t value;

	value = readRegister8(HMC5883L_REG_CONFIG_A);
	value &= 0b11100011;
	value |= (dataRate << 2);

	writeRegister8(HMC5883L_REG_CONFIG_A, value);
}

hmc5883l_dataRate_t Magnetometer::getDataRate(void)
{
	uint8_t value;

	value = readRegister8(HMC5883L_REG_CONFIG_A);
	value &= 0b00011100;
	value >>= 2;

	return (hmc5883l_dataRate_t)value;
}

void Magnetometer::setSamples(hmc5883l_samples_t samples)
{
	uint8_t value;

	value = readRegister8(HMC5883L_REG_CONFIG_A);
	value &= 0b10011111;
	value |= (samples << 5);

	writeRegister8(HMC5883L_REG_CONFIG_A, value);
}

hmc5883l_samples_t Magnetometer::getSamples(void)
{
	uint8_t value;

	value = readRegister8(HMC5883L_REG_CONFIG_A);
	value &= 0b01100000;
	value >>= 5;

	return (hmc5883l_samples_t)value;
}

// Write byte to register
void Magnetometer::writeRegister8(uint8_t reg, uint8_t value)
{
	myWire.beginTransmission(HMC5883L_ADDRESS);
	myWire.write(reg);
	myWire.write(value);
	myWire.endTransmission();
}

// Read byte to register
uint8_t Magnetometer::fastRegister8(uint8_t reg)
{
	uint8_t value = -1;
	#ifndef DISABLE_I2C_MAG
	myWire.beginTransmission(HMC5883L_ADDRESS);
	myWire.write(reg);
	myWire.endTransmission();

	myWire.requestFrom(HMC5883L_ADDRESS, 1);
	value = myWire.read();
	myWire.endTransmission();
	#endif
	return value;
}

// Read byte from register
uint8_t Magnetometer::readRegister8(uint8_t reg)
{
	uint8_t value = -1;
	#ifndef DISABLE_I2C_MAG
	myWire.beginTransmission(HMC5883L_ADDRESS);
	myWire.write(reg);
	myWire.endTransmission();

	myWire.beginTransmission(HMC5883L_ADDRESS);
	myWire.requestFrom(HMC5883L_ADDRESS, 1);
 	while(!myWire.available()) {};
	value = myWire.read();
	myWire.endTransmission();
	#endif

	return value;
}

// Read word from register
int16_t Magnetometer::readRegister16(uint8_t reg)
{
	int16_t value = -1;
	#ifndef DISABLE_I2C_MAG
	myWire.beginTransmission(HMC5883L_ADDRESS);
	myWire.write(reg);
	myWire.endTransmission();

	myWire.beginTransmission(HMC5883L_ADDRESS);
	myWire.requestFrom(HMC5883L_ADDRESS, 2);
	while(!myWire.available()) {};
	uint8_t vha = myWire.read();
	uint8_t vla = myWire.read();
	myWire.endTransmission();

	value = vha << 8 | vla;
	#endif

	return value;
}