#include "power.h"
#include "config.h"
#include "Arduino.h"

power_work_t * work;

void power_setup(power_work_t *wrk)
{
  DPRINT(("power_setup(): setting up power module"));
  pinMode(CHRG_PG, INPUT_PULLUP);
  pinMode(CHRG_CHRG, INPUT_PULLUP);
  pinMode(CHRG_FULL, INPUT_PULLUP);
  pinMode(V_BATT, INPUT);
  pinMode(THERM_BATT, INPUT);
	work = wrk;
};

void power_update()
{

	if(work->state == HAZARD)
	{
		DPRINT(("power_update(): in HAZARD state.. power module halted"));
		return;
	}

	uint8_t pg = !digitalRead(CHRG_PG);
	uint8_t chrg = !digitalRead(CHRG_CHRG);
	uint8_t full = !digitalRead(CHRG_FULL);

	work->vbat = analogRead(V_BATT);
	work->therm_bat = analogRead(THERM_BATT);

	//check temperatures against 
	if(work->therm_bat > HAZARD_HIGH || work->therm_bat < HAZARD_LOW)
		work->state = HAZARD;
	else if(full)
		work->state = FULL;
	else if(chrg)
		work->state = CHARGING;
	else if(work->vbat < LOW_BATT_THRESHOLD)
		work->state = LOWBATT;
	else 
		work->state = DISCHARGING;
    
  DPRINT("power_update(): vbat=");
  DPRINT(work->vbat, DEC);
  DPRINT(" therm_bat=");
  DPRINT(work->therm_bat, DEC);
  DPRINT(" state=");
  DPRINT(work->state, DEC);
  DPRINT("\n");
}



