#include "NewPing.h"
#include "ultrasonicmodule.h"
#include "config.h"
#include "Arduino.h"
    
/* Helper Functions */
void setup_ultrasonic_mux();
void select_sensor(uint8_t n);
UltrasonicModule * me;

UltrasonicModule::UltrasonicModule()  : sonar(US_TRIG, US_ECHO, US_MAX_DISTANCE_CM)
{
  return;
}
void UltrasonicModule::init()
{
  //initialize data structures
  for(int i=0; i < 8; i++)
  {
    filter[i] = MedianFilter(US_MAX_DISTANCE_CM);
    data_m[i] = 0;
  }
 // sonar = NewPing(US_TRIG, US_ECHO, US_MAX_DISTANCE_CM);
  currentSensor = 0;
  go = 0;
  
  //set pin modes
  setup_ultrasonic_mux();
  pinMode(US_ECHO, INPUT);
  pinMode(US_TRIG, OUTPUT);

  me = this;

}

/* copy the data out of the class*/
void UltrasonicModule::getData(uint32_t * data)
{
#ifndef US_TIMER
	for(int i=0; i < 8; i++)
	{
		uint32_t temp;
		select_sensor(currentSensor);
		//delayMicroseconds(500);
		temp = filter[currentSensor].getFilteredValue(sonar.ping());
		*(data+ i) = temp;
		currentSensor = (currentSensor+1) % 8;
	}
#endif

  for(int i=0; i < 8; i++)
  {
    DPRINT("Ultrasonic Read: Sensor ");
    DPRINT(i, DEC);
    DPRINT("gives ");
    DPRINT(data[i], DEC);
    DPRINT("\n");
    
  }
}

/* start the ping loop */
void UltrasonicModule::start()
{
#ifdef US_TIMER
  sonar.ping_timer(handler, US_MAX_DISTANCE_CM);
#endif
  currentSensor = 0;
  go = 1;
}

/* stop the ping loop */
void UltrasonicModule::stop()
{
  go = 0;
#ifdef US_TIMER
  sonar.timer_stop();
#endif
}

void handler_helper()
{
  me->pingHandler();
}

/* handler for timer interrupt used to read ping sensors */
void UltrasonicModule::pingHandler()
{
#ifdef US_TIMER
  if(sonar.check_timer()) //check_timer disables the timer if it returns true
  {
    data[currentSensor] = filter[currentSensor].getFilteredValue(sonar.ping_result)
    if(go) pingNext();
  }
#endif
}

/* advance mux to next sensor, wait a bit, and then ping it */
void UltrasonicModule::pingNext()
{
#ifdef US_TIMER
  currentSensor = (currentSensor+1) % 8;
  select_sensor(currentSensor);
 // delayMicroseconds(5);
  sonar.ping_timer(handler_helper, US_MAX_DISTANCE_CM);
#endif
}
    
/* setup pin modes for the selector mux */
void setup_ultrasonic_mux()
{
  // configure multiplexer pin outs
  //DPRINT(("setup_ultrasonic(): setting mux pin modes\n"));
  pinMode(MUXSEL_0, OUTPUT);
  pinMode(MUXSEL_1, OUTPUT);
  pinMode(MUXSEL_2, OUTPUT);
}
    

/* switch mux bits to select a given sensor */
void select_sensor(uint8_t n)
{
   
  if(n >= 8) 
  {
  //  DPRINT(("Sensor mux out of bounds (%d)\n", n));
    return; 
  }
  else
  {
  //  DPRINT(("Setting sensor mux %d\n", n)); 
    digitalWrite(MUXSEL_0, (n & 0x01)?(HIGH):(LOW));
    digitalWrite(MUXSEL_1, (n & 0x02)?(HIGH):(LOW));
    digitalWrite(MUXSEL_2, (n & 0x04)?(HIGH):(LOW));
  }
  return;  
}

