#include <Arduino.h>
#include "us2.h"
#include "config.h"


//default constructor;
US2::US2()
{


}

void US2::init()
{
      //initialize filters
      pulseComplete = 0;
      pulseWidth = 0;
      currentSensor = 0;
	  ambTemp = 0;

      //initialize data and filters
      for(int i=0; i < 8; i++)
      {
	      filter[i] = MedianFilter(US_MAX_DISTANCE_CM);
	      data[i] = 0;
      }
      pinMode(US_TRIG, OUTPUT);
	  pinMode(US_ECHO, INPUT);
	  pinMode(THERM_AMB, INPUT);
	  
      setupUltrasonicMux();
	  return;
}


// Starts polling loop 
uint32_t US2::start()
{
    //TODO:
    //unregister the current registered instance

    //register my instance
    US2::registeredInstance = this;

    //enable timers
    setupTimers();

    //install irq handler
    enableInterrupts();
	
	currentSensor = 0;
	selectSensor(0);
	ambTemp = adcToTemp(analogRead(THERM_AMB));
	delayMicroseconds(50);
//	delay(20);
    digitalWrite(US_TRIG, 1);
	delayMicroseconds(50);
//    delay(5);
    digitalWrite(US_TRIG, 0);
//    delay(5);
	return 1;
}

float US2::adcToTemp(uint32_t sample)
{
	return sample * 1.0;
}

// Stops polling loop
uint32_t US2::stop()
{
    //TODO:
    //check if I'm the one registerd

    // disable interrupts
    disableInterrupts();

    //unregsiter my instance of the class
    US2::registeredInstance = NULL;
}

//Copy the latest ultrasonic data out of the class
uint32_t US2::getData(uint32_t * dst, uint8_t size)
{
   uint8_t i;
   for(i=0; i < size && i<8; i++)
    *(dst+i) = filter[i].getMedian();
   return i;
}

uint32_t  US2::gotIRQ(uint32_t sample)
{
	filter[currentSensor].addSample(sample);
    currentSensor = (currentSensor+1)%8;
	selectSensor(currentSensor);
    delayMicroseconds(200);
	//ambTemp = adcToTemp(analogRead(THERM_AMB));
    digitalWrite(US_TRIG, 1);
	delayMicroseconds(100);
  //  delay(1);
    digitalWrite(US_TRIG, 0);
		delayMicroseconds(50);
  //  delay(1);
    return 0;
};




/* switch mux bits to select a given sensor */
void US2::selectSensor(uint8_t n)
{
   
  if(n >= 8) 
  {
    //DPRINT(("Sensor mux out of bounds (%d)\n", n));
    return; 
  }
  else
  {
    //DPRINT(("Setting sensor mux %d\n", n)); 
    digitalWrite(MUXSEL_0, (n & 0x01)?(HIGH):(LOW));
    digitalWrite(MUXSEL_1, (n & 0x02)?(HIGH):(LOW));
    digitalWrite(MUXSEL_2, (n & 0x04)?(HIGH):(LOW));
  }
  return;  
}

void US2::enableInterrupts()
{
  attachInterrupt(6, TC4_Handler, HIGH);                                        // Attach interrupts to digital pin 12 (external interrupt 3)
  NVIC_SetPriority(TC4_IRQn, 0);      // Set the Nested Vector Interrupt Controller (NVIC) priority for TC3 to 0 (highest)
  NVIC_EnableIRQ(TC4_IRQn);           // Connect the TC3 timer to the Nested Vector Interrupt Controller (NVIC)
  	//TC4->COUNT16.INTENCLR.bit.ERR = 1;
}

void US2::disableInterrupts()
{
  NVIC_DisableIRQ(TC4_IRQn);
  NVIC_ClearPendingIRQ(TC4_IRQn);
}

void US2::setupTimers()
{
  REG_PM_APBCMASK |= PM_APBCMASK_EVSYS;     // Switch on the event system peripheral
  
  REG_GCLK_GENDIV = GCLK_GENDIV_DIV(1) |    // Divide the 48MHz system clock by 3 = 16MHz
                    GCLK_GENDIV_ID(5);      // Set division on Generic Clock Generator (GCLK) 5
  while (GCLK->STATUS.bit.SYNCBUSY);        // Wait for synchronization

  REG_GCLK_GENCTRL = GCLK_GENCTRL_IDC |           // Set the duty cycle to 50/50 HIGH/LOW
                     GCLK_GENCTRL_GENEN |         // Enable GCLK 5
                     GCLK_GENCTRL_SRC_DFLL48M |   // Set the clock source to 48MHz 
                     GCLK_GENCTRL_ID(5);          // Set clock source on GCLK 5
  while (GCLK->STATUS.bit.SYNCBUSY);              // Wait for synchronization*/

  REG_GCLK_CLKCTRL = GCLK_CLKCTRL_CLKEN |         // Enable the generic clock...
                     GCLK_CLKCTRL_GEN_GCLK5 |     // ....on GCLK5
                     GCLK_CLKCTRL_ID_TC4_TC5;    // Feed the GCLK5 to TCC2 and TC3
  while (GCLK->STATUS.bit.SYNCBUSY);              // Wait for synchronization

  REG_EIC_EVCTRL |= EIC_EVCTRL_EXTINTEO4;                                 // Enable event output on external interrupt 3

  
  REG_EVSYS_USER = EVSYS_USER_CHANNEL(1) |                                // Attach the event user (receiver) to channel 0 (n + 1)
                   EVSYS_USER_USER(EVSYS_ID_USER_TC4_EVU);                // Set the event user (receiver) as timer TC4

  REG_EVSYS_CHANNEL = EVSYS_CHANNEL_EDGSEL_NO_EVT_OUTPUT |                // No event edge detection
                      EVSYS_CHANNEL_PATH_ASYNCHRONOUS |                   // Set event path as asynchronous
                      EVSYS_CHANNEL_EVGEN(EVSYS_ID_GEN_EIC_EXTINT_4) |    // Set event generator (sender) as external interrupt 4
                      EVSYS_CHANNEL_CHANNEL(0);                           // Attach the generator (sender) to channel 0

  REG_TC4_EVCTRL |= TC_EVCTRL_TCEI |              // Enable the TC event input
                    /*TC_EVCTRL_TCINV |*/         // Invert the event input
                    TC_EVCTRL_EVACT_PPW;          // Set up the timer for capture: CC0 period, CC1 pulsewidth
                   
  REG_TC4_READREQ = TC_READREQ_RREQ |             // Enable a read request
                    TC_READREQ_ADDR(0x06);        // Offset of the CTRLC register
  while (TC4->COUNT16.STATUS.bit.SYNCBUSY);       // Wait for (read) synchronization
  REG_TC4_CTRLC |= TC_CTRLC_CPTEN1 |              // Enable capture on CC1
                   TC_CTRLC_CPTEN0;               // Enable capture on CC0
  while (TC4->COUNT16.STATUS.bit.SYNCBUSY);       // Wait for (write) synchronization

  //NVIC_DisableIRQ(TC4_IRQn);
  //NVIC_ClearPendingIRQ(TC4_IRQn);
//  NVIC_SetPriority(TC4_IRQn, 0);      // Set the Nested Vector Interrupt Controller (NVIC) priority for TC3 to 0 (highest)
//  NVIC_EnableIRQ(TC4_IRQn);           // Connect the TC4 timer to the Nested Vector Interrupt Controller (NVIC)
 
  REG_TC4_INTENSET = TC_INTENSET_MC1 | TC_INTENSET_OVF|         // Enable compare channel 1 (CC1) interrupts
                     TC_INTENSET_MC0 ;             // Enable compare channel 0 (CC0) interrupts
  
  REG_TC4_CTRLA |= TC_CTRLA_PRESCALER_DIV16 |     // Set prescaler to 16, 16MHz/16 = 1MHz
                   TC_CTRLA_ENABLE;               // Enable TC3
  while (TC4->COUNT16.STATUS.bit.SYNCBUSY);       // Wait for synchronization

}

US2* US2::registeredInstance = NULL;

static volatile uint32_t sample;

void TC4_Handler()                                // Interrupt Service Routine (ISR) for timer TC3
{     
  uint32_t temp;
  // Check for match counter 0 (MC0) interrupt
  if (TC4->COUNT16.INTFLAG.bit.MC0)             
  {
    REG_TC4_READREQ = TC_READREQ_RREQ |           // Enable a read request
                      TC_READREQ_ADDR(0x18);      // Offset address of the CC0 register
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);     // Wait for (read) synchronization
    temp = REG_TC4_COUNT16_CC0;              // Copy the period  
    //registeredInstance->periodComplete = true;                        // Indicate that the period is complete
	//  US2::registeredInstance->gotIRQ(sample);
  //US2::registeredInstance->gotIRQ(sample);
  }

  // Check for match counter 1 (MC1) interrupt
  if (TC4->COUNT16.INTFLAG.bit.MC1)           
  {
    //if(TC4->COUNT16.INTFLAG.bit.OVF) TC4->COUNT16.INTFLAG.bit.OVF = 1;
	REG_TC4_READREQ = TC_READREQ_RREQ |           // Enable a read request
                      TC_READREQ_ADDR(0x1A);      // Offset address of the CC1 register
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);     // Wait for (read) synchronization
    sample = REG_TC4_COUNT16_CC1;          // Copy the pulse-width
    US2::registeredInstance->gotIRQ(sample);


  }
  
  if(TC4->COUNT16.INTFLAG.bit.OVF)
  {
	TC4->COUNT16.INTFLAG.bit.OVF = 1;
	//REG_TC4_READREQ = TC_READREQ_RREQ |           // Enable a read request
    //                  TC_READREQ_ADDR(0x1A);      // Offset address of the CC1 register
    //while (TC4->COUNT16.STATUS.bit.SYNCBUSY);     // Wait for (read) synchronization
    //sample = 0;        // Copy the pulse-width
		REG_TC4_READREQ = TC_READREQ_RREQ |           // Enable a read request
                      TC_READREQ_ADDR(0x1A);      // Offset address of the CC1 register
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);     // Wait for (read) synchronization
    sample = REG_TC4_COUNT16_CC1;  
   US2::registeredInstance->gotIRQ(0);
   //sample = 0;
  }
  
    if(TC4->COUNT16.INTFLAG.bit.ERR)
  {
	TC4->COUNT16.INTFLAG.bit.ERR = 1;
	//REG_TC4_READREQ = TC_READREQ_RREQ |           // Enable a read request
    //                  TC_READREQ_ADDR(0x1A);      // Offset address of the CC1 register
    //while (TC4->COUNT16.STATUS.bit.SYNCBUSY);     // Wait for (read) synchronization
    //sample = (1<<16)-1;        // Copy the pulse-width
//    US2::registeredInstance->gotIRQ(sample);
  }
  //  if(TC4->COUNT16.INTFLAG.bit.SYNCRDY)
//  {
//	TC4->COUNT16.INTFLAG.bit.SYNCRDY = 1;
	//REG_TC4_READREQ = TC_READREQ_RREQ |           // Enable a read request
    //                  TC_READREQ_ADDR(0x1A);      // Offset address of the CC1 register
    //while (TC4->COUNT16.STATUS.bit.SYNCBUSY);     // Wait for (read) synchronization
    //sample = (1<<16)-1;        // Copy the pulse-width
//    US2::registeredInstance->gotIRQ(sample);
//  }
  


 // US2::registeredInstance->gotIRQ(sample);

}
    
/* setup pin modes for the selector mux */
void US2::setupUltrasonicMux()
{
  // configure multiplexer pin outs
  DPRINT(("setup_ultrasonic(): setting mux pin modes\n"));
  pinMode(MUXSEL_0, OUTPUT);
  pinMode(MUXSEL_1, OUTPUT);
  pinMode(MUXSEL_2, OUTPUT);
}

